@@ -180,7 +180,7 @@ main(int argc, char **argv)
 	 */
 	putenv ("BONOBO_ACTIVATION_DEBUG_OUTPUT=1");
 
-	gnome_program_init ("panel", VERSION,
+	gnome_program_init ("gnome-panel", VERSION,
 			    LIBGNOMEUI_MODULE,
 			    argc, argv,
 			    GNOME_PARAM_POPT_TABLE, options,
@@ -197,22 +197,26 @@ main(int argc, char **argv)
 
 	client = gnome_master_client ();
 
-	/* FIXME - get the session stuff working properly */
+	panel_session_set_restart_command (client, argv [0]);
 
-	g_signal_connect (G_OBJECT (client), "save_yourself",
+	g_signal_connect (client, "save_yourself",
 			  G_CALLBACK (panel_session_save), argv[0]);
-	g_signal_connect (G_OBJECT (client), "die",
+	g_signal_connect (client, "die",
 			  G_CALLBACK (panel_session_die), NULL);
 
+	panel_register_window_icon ();
+
 	panel_tooltips = gtk_tooltips_new ();
 
 	xstuff_init ();
 	multiscreen_init ();
 
 	panel_gconf_add_dir ("/apps/panel/global");
-	panel_gconf_add_dir ("/desktop/gnome/menus");
+	panel_gconf_add_dir ("/desktop/gnome/interface");
 	panel_gconf_notify_add ("/apps/panel/global", panel_global_config_notify, NULL);
 
+	status_applet_create_offscreen ();
+
 	session_load ();	
 
 	kill_free_drawers ();
@@ -230,8 +234,6 @@ main(int argc, char **argv)
 	/* add some timeouts */
 	g_timeout_add (10*1000, menu_age_timeout, NULL);
 
-	status_applet_create_offscreen ();
-
 	gtk_main ();
 
 	return 0;
